from sensenova.api_resources.abstract.api_resource import APIResource
from sensenova.api_resources.abstract.listable_api_resource import ListableAPIResource
from sensenova.api_resources.abstract.createable_api_resource import CreatableAPIResource
from sensenova.api_resources.abstract.deletable_api_resource import DeletableAPIResource
from sensenova.api_resources.abstract.engine_api_resource import EngineAPIResource
from sensenova.api_resources.abstract.fileable_api_resource import FileableAPIResource
from sensenova.api_resources.abstract.downloadable_api_resource import DownloadableAPIResource
from sensenova.api_resources.abstract.uploadable_api_resource import UploadableAPIResource